@@ -1,7 +1,6 @@
 entry_points = {
     'SSHBaseForwarder': [
         'base = ssh_proxy_server.forwarders.ssh:SSHForwarder',
-        'injectorshell = ssh_proxy_server.plugins.ssh.injectorshell:SSHInjectableForwarder',
         'mirrorshell = ssh_proxy_server.plugins.ssh.mirrorshell:SSHMirrorForwarder',
         'noshell = ssh_proxy_server.plugins.ssh.noshell:NoShellForwarder',
         'sessionlogger = ssh_proxy_server.plugins.ssh.sessionlogger:SSHLogForwarder'
@@ -23,6 +22,9 @@ entry_points = {
     'BaseServerInterface': [
         'base = ssh_proxy_server.interfaces.server:ServerInterface'
     ],
+    'BaseSSHProxyManager': [
+        'base = ssh_proxy_server.interfaces.server:SSHProxyManager'
+    ],
     'Authenticator': [
         'passthrough = ssh_proxy_server.authentication:AuthenticatorPassThrough'
     ]
