@@ -14,7 +14,6 @@ from ssh_proxy_server.interfaces import (
     BaseServerInterface,
     ServerInterface
 )
-from ssh_proxy_server.forwarders.base import BaseForwarder
 from ssh_proxy_server.forwarders.scp import (
     SCPBaseForwarder,
     SCPForwarder
@@ -32,6 +31,8 @@ from ssh_proxy_server.interfaces.sftp import (
     SFTPProxyServerInterface
 )
 
+
+from ssh_proxy_server.workarounds import dropbear
 from ssh_proxy_server.plugins.ssh.mirrorshell import SSHMirrorForwarder
 
 
@@ -62,7 +63,7 @@ def main():
         '--host-key-algorithm',
         dest='host_key_algorithm',
         default='rsa',
-        choices=['dss', 'rsa', 'ecdsa'],
+        choices=['dss', 'rsa', 'ecdsa', 'ed25519'],
         help='host key algorithm (default rsa)'
     )
     parser.add_argument(
@@ -138,9 +139,18 @@ def main():
         choices=['warning', 'info', 'debug'],
         help='set paramikos log level'
     )
+    parser.add_argument(
+        '--disable-workarounds',
+        dest='disable_workarounds',
+        action='store_true',
+        help='disable paramiko workarounds'
+    )
 
     args = parser.parse_args()
 
+    if not args.disable_workarounds:
+        Transport.run = dropbear.transport_run
+
     if args.paramiko_log_level == 'debug':
         logging.getLogger("paramiko").setLevel(logging.DEBUG)
     elif args.paramiko_log_level == 'info':
@@ -164,7 +174,8 @@ def main():
         sftp_handler=args.sftp_handler,
         authentication_interface=args.auth_interface,
         authenticator=args.authenticator,
-        transparent=args.transparent
+        transparent=args.transparent,
+        args=args
     )
     if args.banner_name is not None:
         Transport._CLIENT_ID = args.banner_name
